@@ -1,7 +1,8 @@
 #!/usr/bin/python
+# -*- coding: utf-8 -*-
 
 # JabberBot: A simple jabber/xmpp bot framework
-# Copyright (c) 2007-2010 Thomas Perl <thpinfo.com>
+# Copyright (c) 2007-2011 Thomas Perl <thp.io/about>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,6 +18,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+"""A simple framework for creating Jabber/XMPP bots and services"""
 
 import os
 import re
@@ -27,14 +29,15 @@ try:
 except ImportError:
     print >>sys.stderr, 'You need to install xmpppy from http://xmpppy.sf.net/.'
     sys.exit(-1)
+
+import time
 import inspect
+import logging
 import traceback
 
-"""A simple jabber/xmpp bot framework"""
-
-__author__ = 'Thomas Perl <thp@thpinfo.com>'
-__version__ = '0.10'
-__website__ = 'http://thpinfo.com/2007/python-jabberbot/'
+__author__ = 'Thomas Perl <m@thp.io>'
+__version__ = '0.13'
+__website__ = 'http://thp.io/2007/python-jabberbot/'
 __license__ = 'GPLv3 or later'
 
 def botcmd(*args, **kwargs):
@@ -58,9 +61,25 @@ class JabberBot(object):
     MSG_AUTHORIZE_ME = 'Hey there. You are not yet on my roster. Authorize my request and I will do the same.'
     MSG_NOT_AUTHORIZED = 'You did not authorize my subscription request. Access denied.'
 
-    def __init__(self, username, password, res=None, debug=False):
-        """Initializes the jabber bot and sets up commands."""
+    PING_FREQUENCY = 0 # Set to the number of seconds, e.g. 60.
+    PING_TIMEOUT = 2 # Seconds to wait for a response.
+
+    def __init__(self, username, password, res=None, debug=False,
+            privatedomain=False, acceptownmsgs=False):
+        """Initializes the jabber bot and sets up commands.
+
+        If privatedomain is provided, it should be either
+        True to only allow subscriptions from the same domain
+        as the bot or a string that describes the domain for
+        which subscriptions are accepted (e.g. 'jabber.org').
+
+        If acceptownmsgs it set to True, this bot will accept
+        messages from the same JID that the bot itself has. This
+        is useful when using JabberBot with a single Jabber account
+        and multiple instances that want to talk to each other.
+        """
         self.__debug = debug
+        self.log = logging.getLogger(__name__)
         self.__username = username
         self.__password = password
         self.jid = xmpp.JID(self.__username)
@@ -71,14 +90,19 @@ class JabberBot(object):
         self.__status = None
         self.__seen = {}
         self.__threads = {}
+        self.__lastping = None
+        self.__privatedomain = privatedomain
+        self.__acceptownmsgs = acceptownmsgs
 
         self.commands = {}
         for name, value in inspect.getmembers(self):
             if inspect.ismethod(value) and getattr(value, '_jabberbot_command', False):
                 name = getattr(value, '_jabberbot_command_name')
-                self.debug('Registered command: %s' % name)
+                self.log.debug('Registered command: %s' % name)
                 self.commands[name] = value
 
+        self.roster = None
+
 ################################
 
     def _send_status(self):
@@ -106,13 +130,6 @@ class JabberBot(object):
 
 ################################
 
-    def debug(self, s):
-        if self.__debug: self.log(s)
-
-    def log( self, s):
-        """Logging facility, can be overridden in subclasses to log to file, etc.."""
-        print self.__class__.__name__, ':', s
-
     def connect( self):
         if not self.conn:
             if self.__debug:
@@ -122,27 +139,27 @@ class JabberBot(object):
 
             conres = conn.connect()
             if not conres:
-                self.log( 'unable to connect to server %s.' % self.jid.getDomain())
+                self.log.error('unable to connect to server %s.' % self.jid.getDomain())
                 return None
             if conres<>'tls':
-                self.log("Warning: unable to establish secure connection - TLS failed!")
+                self.log.warning('unable to establish secure connection - TLS failed!')
 
             authres = conn.auth(self.jid.getNode(), self.__password, self.res)
             if not authres:
-                self.log('unable to authorize with server.')
+                self.log.error('unable to authorize with server.')
                 return None
             if authres<>'sasl':
-                self.log("Warning: unable to perform SASL auth os %s. Old authentication method used!" % self.jid.getDomain())
+                self.log.warning("unable to perform SASL auth os %s. Old authentication method used!" % self.jid.getDomain())
 
-            conn.RegisterHandler('message', self.callback_message)
-            conn.RegisterHandler('presence', self.callback_presence)
             conn.sendInitPresence()
             self.conn = conn
             self.roster = self.conn.Roster.getRoster()
-            self.log('*** roster ***')
+            self.log.info('*** roster ***')
             for contact in self.roster.getItems():
-                self.log('  ' + str(contact))
-            self.log('*** roster ***')
+                self.log.info('  %s' % contact)
+            self.log.info('*** roster ***')
+            self.conn.RegisterHandler('message', self.callback_message)
+            self.conn.RegisterHandler('presence', self.callback_presence)
 
         return self.conn
 
@@ -150,7 +167,7 @@ class JabberBot(object):
         """Join the specified multi-user chat room"""
         if username is None:
             username = self.__username.split('@')[0]
-        my_room_JID = '/'.join(room, username)
+        my_room_JID = '/'.join((room, username))
         self.connect().send(xmpp.Presence(to=my_room_JID))
 
     def quit( self):
@@ -265,11 +282,11 @@ class JabberBot(object):
 
     def status_type_changed(self, jid, new_status_type):
         """Callback for tracking status types (available, away, offline, ...)"""
-        self.debug('user %s changed status to %s' % (jid, new_status_type))
+        self.log.debug('user %s changed status to %s' % (jid, new_status_type))
 
     def status_message_changed(self, jid, new_status_message):
         """Callback for tracking status messages (the free-form status text)"""
-        self.debug('user %s updated text to %s' % (jid, new_status_message))
+        self.log.debug('user %s updated text to %s' % (jid, new_status_message))
 
     def broadcast(self, message, only_available=False):
         """Broadcast a message to all users 'seen' by this bot.
@@ -281,11 +298,12 @@ class JabberBot(object):
                 self.send(jid, message)
 
     def callback_presence(self, conn, presence):
+        self.__lastping = time.time()
         jid, type_, show, status = presence.getFrom(), \
                 presence.getType(), presence.getShow(), \
                 presence.getStatus()
 
-        if self.jid.bareMatch(jid):
+        if self.jid.bareMatch(jid) and not self.__acceptownmsgs:
             # Ignore our own presence messages
             return
 
@@ -305,15 +323,33 @@ class JabberBot(object):
             self.status_type_changed(jid, self.OFFLINE)
 
         try:
-            subscription = self.roster.getSubscription(str(jid))
-        except KeyError, ke:
+            subscription = self.roster.getSubscription(unicode(jid.__str__()))
+        except KeyError, e:
             # User not on our roster
             subscription = None
+        except AttributeError, e:
+            # Recieved presence update before roster built
+            return
 
         if type_ == 'error':
-            self.log(presence.getError())
+            self.log.error(presence.getError())
 
-        self.debug('Got presence: %s (type: %s, show: %s, status: %s, subscription: %s)' % (jid, type_, show, status, subscription))
+        self.log.debug('Got presence: %s (type: %s, show: %s, status: %s, subscription: %s)' % (jid, type_, show, status, subscription))
+
+        # If subscription is private, disregard anything not from the private domain
+        if self.__privatedomain and type_ in ('subscribe', 'subscribed', 'unsubscribe'):
+            if self.__privatedomain == True:
+                # Use the bot's domain
+                domain = self.jid.getDomain()
+            else:
+                # Use the specified domain
+                domain = self.__privatedomain
+
+            # Check if the sender is in the private domain
+            user_domain = jid.getDomain()
+            if domain != user_domain:
+                self.log.info('Ignoring subscribe request: %s does not match private domain (%s)' % (user_domain, domain))
+                return
 
         if type_ == 'subscribe':
             # Incoming presence subscription request
@@ -336,6 +372,7 @@ class JabberBot(object):
 
     def callback_message( self, conn, mess):
         """Messages sent to the bot will arrive here. Command handling + routing is done in this function."""
+        self.__lastping = time.time()
 
         # Prepare to handle either private chats or group chats
         type     = mess.getType()
@@ -345,14 +382,14 @@ class JabberBot(object):
         username = self.get_sender_username(mess)
 
         if type not in ("groupchat", "chat"):
-            self.debug("unhandled message type: %s" % type)
+            self.log.debug("unhandled message type: %s" % type)
             return
 
-        self.debug("*** props = %s" % props)
-        self.debug("*** jid = %s" % jid)
-        self.debug("*** username = %s" % username)
-        self.debug("*** type = %s" % type)
-        self.debug("*** text = %s" % text)
+        self.log.debug("*** props = %s" % props)
+        self.log.debug("*** jid = %s" % jid)
+        self.log.debug("*** username = %s" % username)
+        self.log.debug("*** type = %s" % type)
+        self.log.debug("*** text = %s" % text)
 
         # Ignore messages from before we joined
         if xmpp.NS_DELAY in props: return
@@ -365,8 +402,8 @@ class JabberBot(object):
 
         # Ignore messages from users not seen by this bot
         if jid not in self.__seen:
-            self.log('Ignoring message from unseen guest: %s' % jid)
-            self.debug("I've seen: %s" % ["%s" % x for x in self.__seen.keys()])
+            self.log.info('Ignoring message from unseen guest: %s' % jid)
+            self.log.debug("I've seen: %s" % ["%s" % x for x in self.__seen.keys()])
             return
 
         # Remember the last-talked-in thread for replies
@@ -377,15 +414,14 @@ class JabberBot(object):
         else:
             command, args = text, ''
         cmd = command.lower()
-        self.debug("*** cmd = %s" % cmd)
+        self.log.debug("*** cmd = %s" % cmd)
 
         if self.commands.has_key(cmd):
             try:
                 reply = self.commands[cmd](mess, args)
             except Exception, e:
                 reply = traceback.format_exc(e)
-                self.log('An error happened while processing a message ("%s") from %s: %s"' % (text, jid, reply))
-                print reply
+                self.log.exception('An error happened while processing a message ("%s") from %s: %s"' % (text, jid, reply))
         else:
             # In private chat, it's okay for the bot to always respond.
             # In group chat, the bot should silently ignore commands it
@@ -438,12 +474,15 @@ class JabberBot(object):
             else:
                 description = 'Available commands:'
 
-            usage = '\n'.join(sorted(['%s: %s' % (name, (command.__doc__ or '(undocumented)').split('\n', 1)[0]) for (name, command) in self.commands.items() if name != 'help' and not command._jabberbot_hidden]))
+            usage = '\n'.join(sorted([
+                '%s: %s' % (name, (command.__doc__.strip() or '(undocumented)').split('\n', 1)[0])
+                for (name, command) in self.commands.iteritems() if name != 'help' and not command._jabberbot_hidden
+            ]))
             usage = usage + '\n\nType help <command name> to get more info about that specific command.'
         else:
             description = ''
             if args in self.commands:
-                usage = self.commands[args].__doc__ or 'undocumented'
+                usage = self.commands[args].__doc__.strip() or 'undocumented'
             else:
                 usage = 'That command is not defined.'
 
@@ -456,7 +495,29 @@ class JabberBot(object):
 
     def idle_proc( self):
         """This function will be called in the main loop."""
-        pass
+        self._idle_ping()
+
+    def _idle_ping(self):
+        """Pings the server, calls on_ping_timeout() on no response.
+
+        To enable set self.PING_FREQUENCY to a value higher than zero.
+        """
+        if self.PING_FREQUENCY and time.time() - self.__lastping > self.PING_FREQUENCY:
+            self.__lastping = time.time()
+            #logging.debug('Pinging the server.')
+            ping = xmpp.Protocol('iq',typ='get',payload=[xmpp.Node('ping',attrs={'xmlns':'urn:xmpp:ping'})])
+            try:
+                res = self.conn.SendAndWaitForResponse(ping, self.PING_TIMEOUT)
+                #logging.debug('Got response: ' + str(res))
+                if res is None:
+                    self.on_ping_timeout()
+            except IOError, e:
+                logging.error('Error pinging the server: %s, treating as ping timeout.' % e)
+                self.on_ping_timeout()
+
+    def on_ping_timeout(self):
+        logging.info('Terminating due to PING timeout.')
+        self.quit()
 
     def shutdown(self):
         """This function will be called when we're done serving
@@ -470,20 +531,21 @@ class JabberBot(object):
         """Connects to the server and handles messages."""
         conn = self.connect()
         if conn:
-            self.log('bot connected. serving forever.')
+            self.log.info('bot connected. serving forever.')
         else:
-            self.log('could not connect to server - aborting.')
+            self.log.warn('could not connect to server - aborting.')
             return
 
         if connect_callback:
             connect_callback()
+        self.__lastping = time.time()
 
         while not self.__finished:
             try:
                 conn.Process(1)
                 self.idle_proc()
             except KeyboardInterrupt:
-                self.log('bot stopped by user request. shutting down.')
+                self.log.info('bot stopped by user request. shutting down.')
                 break
 
         self.shutdown()
